@@ -2,6 +2,7 @@
 from AccessControl.ZopeGuards import guarded_getattr
 from Acquisition import aq_base
 from DateTime import DateTime
+from io import BytesIO
 from plone.memoize import ram
 from plone.namedfile.file import FILECHUNK_CLASSES
 from plone.namedfile.interfaces import IAvailableSizes
@@ -15,9 +16,11 @@ from plone.scale.interfaces import IImageScaleFactory
 from plone.scale.interfaces import IScaledImageQuality
 from plone.scale.scale import scaleImage
 from plone.scale.storage import AnnotationStorage
+from Products.CMFPlone.utils import safe_encode
 from Products.Five import BrowserView
 from xml.sax.saxutils import quoteattr
 from zExceptions import Unauthorized
+from ZODB.blob import BlobFile
 from ZODB.POSException import ConflictError
 from zope.component import queryUtility
 from zope.deprecation import deprecate
@@ -29,6 +32,7 @@ from zope.traversing.interfaces import ITraversable
 from zope.traversing.interfaces import TraversalError
 
 import logging
+import six
 
 
 logger = logging.getLogger(__name__)
@@ -42,7 +46,7 @@ class ImageScale(BrowserView):
     # protected
     # (it's okay because we explicitly validate access to the image attribute
     # when we retrieve it)
-    __roles__ = ('Anonymous',)
+    __roles__ = ("Anonymous",)
     __allow_access_to_unprotected_subobjects__ = 1
     data = None
 
@@ -54,46 +58,47 @@ class ImageScale(BrowserView):
             self.data = getattr(self.context, self.fieldname)
 
         url = self.context.absolute_url()
-        major, minor = self.data.contentType.split('/')
-        if minor == 'svg+xml':
-            # handle special case of svg:
-            extension = '.svg'
+        extension = self.data.contentType.split("/")[-1].lower()
+        if self.data.contentType == "image/svg+xml":
+            extension = "svg"
+        if "uid" in info:
+            name = info["uid"]
         else:
-            extension = '.' + minor.lower()
-        if 'uid' in info:
-            name = info['uid']
-        else:
-            name = info['fieldname']
-        self.__name__ = u'{0}{1}'.format(name, extension)
-        self.url = u'{0}/@@images/{1}'.format(url, self.__name__)
-        self.srcset = info.get('srcset', [])
+            name = info["fieldname"]
+        self.__name__ = u"{0}.{1}".format(name, extension)
+        self.url = u"{0}/@@images/{1}".format(url, self.__name__)
+        self.srcset = info.get("srcset", [])
 
     def absolute_url(self):
         return self.url
 
     def srcset_attribute(self):
         _srcset_attr = []
-        extension = self.data.contentType.split('/')[-1].lower()
+        extension = self.data.contentType.split("/")[-1].lower()
         for scale in self.srcset:
             _srcset_attr.append(
-                u'{0}/@@images/{1}.{2} {3}x'.format(
-                    self.context.absolute_url(),
-                    scale['uid'],
-                    extension,
-                    scale['scale'],
+                u"{0}/@@images/{1}.{2} {3}x".format(
+                    self.context.absolute_url(), scale["uid"], extension, scale["scale"]
                 )
             )
-        srcset_attr = ', '.join(_srcset_attr)
+        srcset_attr = ", ".join(_srcset_attr)
         return srcset_attr
 
-    def tag(self, height=_marker, width=_marker, alt=_marker,
-            css_class=None, title=_marker, **kwargs):
+    def tag(
+        self,
+        height=_marker,
+        width=_marker,
+        alt=_marker,
+        css_class=None,
+        title=_marker,
+        **kwargs
+    ):
         """Create a tag including scale
         """
         if height is _marker:
-            height = getattr(self, 'height', self.data._height)
+            height = getattr(self, "height", self.data._height)
         if width is _marker:
-            width = getattr(self, 'width', self.data._width)
+            width = getattr(self, "width", self.data._width)
 
         if alt is _marker:
             alt = self.context.Title()
@@ -101,36 +106,35 @@ class ImageScale(BrowserView):
             title = self.context.Title()
 
         values = [
-            ('src', self.url),
-            ('alt', alt),
-            ('title', title),
-            ('height', height),
-            ('width', width),
-            ('class', css_class),
+            ("src", self.url),
+            ("alt", alt),
+            ("title", title),
+            ("height", height),
+            ("width", width),
+            ("class", css_class),
         ]
 
         srcset_attr = self.srcset_attribute()
         if srcset_attr:
-            values.append(('srcset', srcset_attr))
+            values.append(("srcset", srcset_attr))
 
         values.extend(kwargs.items())
 
-        parts = ['<img']
+        parts = ["<img"]
         for k, v in values:
             if v is None:
                 continue
             if isinstance(v, int):
                 v = str(v)
-            elif isinstance(v, str):
-                v = unicode(v, 'utf8')
-            parts.append(u'{0}={1}'.format(k, quoteattr(v)))
-        parts.append('/>')
+            elif isinstance(v, six.binary_type):
+                v = six.text_type(v, "utf8")
+            parts.append(u"{0}={1}".format(k, quoteattr(v)))
+        parts.append("/>")
 
-        return u' '.join(parts)
+        return u" ".join(parts)
 
     def validate_access(self):
-        fieldname = getattr(self.data, 'fieldname',
-                            getattr(self, 'fieldname', None))
+        fieldname = getattr(self.data, "fieldname", getattr(self, "fieldname", None))
         guarded_getattr(self.context, fieldname)
 
     def index_html(self):
@@ -157,20 +161,19 @@ class ImageScale(BrowserView):
         """
         self.validate_access()
         set_headers(self.data, REQUEST.response)
-        return ''
+        return ""
 
-    HEAD.__roles__ = ('Anonymous',)
+    HEAD.__roles__ = ("Anonymous",)
 
 
 @implementer(ITraversable)
 class ImmutableTraverser(object):
-
     def __init__(self, scale):
         self.scale = scale
 
     def traverse(self, name, furtherPath):
         if furtherPath:
-            raise TraversalError('Do not know how to handle further path')
+            raise TraversalError("Do not know how to handle further path")
         else:
             if self.scale:
                 return self.scale.tag()
@@ -180,7 +183,6 @@ class ImmutableTraverser(object):
 
 @implementer(IImageScaleFactory)
 class DefaultImageScalingFactory(object):
-
     def __init__(self, context):
         self.context = context
 
@@ -193,21 +195,17 @@ class DefaultImageScalingFactory(object):
 
     def create_scale(self, data, direction, height, width, **parameters):
         return scaleImage(
-            data,
-            direction=direction,
-            height=height,
-            width=width,
-            **parameters
+            data, direction=direction, height=height, width=width, **parameters
         )
 
     def __call__(
-            self,
-            fieldname=None,
-            direction='thumbnail',
-            height=None,
-            width=None,
-            scale=None,
-            **parameters
+        self,
+        fieldname=None,
+        direction="thumbnail",
+        height=None,
+        width=None,
+        scale=None,
+        **parameters
     ):
 
         """Factory for image scales`.
@@ -217,16 +215,15 @@ class DefaultImageScalingFactory(object):
             return
 
         if height is None and width is None:
-            dummy, format_ = orig_value.contentType.split('/', 1)
+            dummy, format_ = orig_value.contentType.split("/", 1)
             return None, format_, (orig_value._width, orig_value._height)
         orig_data = None
         try:
             orig_data = orig_value.open()
         except AttributeError:
-            orig_data = getattr(aq_base(orig_value), 'data', orig_value)
+            orig_data = getattr(aq_base(orig_value), "data", orig_value)
         if not orig_data:
             return
-
         # Handle cases where large image data is stored in FileChunks instead
         # of plain string
         if isinstance(orig_data, tuple(FILECHUNK_CLASSES)):
@@ -236,15 +233,13 @@ class DefaultImageScalingFactory(object):
 
         # If quality wasn't in the parameters, try the site's default scaling
         # quality if it exists.
-        if 'quality' not in parameters:
+        if "quality" not in parameters:
             quality = self.get_quality()
             if quality:
-                parameters['quality'] = quality
+                parameters["quality"] = quality
 
-        try:
-            if getattr(orig_value, 'contentType', '') == 'image/svg+xml':
-                result = orig_data.read(), 'SVG+XML', (width, height)
-            else:
+        if not getattr(orig_value, "contentType", "") == "image/svg+xml":
+            try:
                 result = self.create_scale(
                     orig_data,
                     direction=direction,
@@ -252,41 +247,53 @@ class DefaultImageScalingFactory(object):
                     width=width,
                     **parameters
                 )
-        except (ConflictError, KeyboardInterrupt):
-            raise
-        except Exception:
-            logger.exception(
-                'Could not scale "{0!r}" of {1!r}'.format(
-                    orig_value,
-                    self.context.absolute_url,
-                ),
-            )
-            return
+            except (ConflictError, KeyboardInterrupt):
+                raise
+            except Exception:
+                logger.exception(
+                    'Could not scale "{0!r}" of {1!r}'.format(
+                        orig_value, self.context.absolute_url(),
+                    ),
+                )
+                return
+            if result is None:
+                return
+        else:
+            if isinstance(orig_data, (six.text_type)):
+                orig_data = safe_encode(orig_data)
+            if isinstance(orig_data, (bytes)):
+                orig_data = BytesIO(orig_data)
 
-        if result is None:
-            return
+            result = orig_data.read(), "svg+xml", (width, height)
 
         data, format_, dimensions = result
-        mimetype = 'image/{0}'.format(format_.lower())
+        mimetype = "image/{0}".format(format_.lower())
         value = orig_value.__class__(
-            data,
-            contentType=mimetype,
-            filename=orig_value.filename,
+            data, contentType=mimetype, filename=orig_value.filename,
         )
         value.fieldname = fieldname
+
+        # make sure the file is closed to avoid error:
+        # ZODB-5.5.1-py3.7.egg/ZODB/blob.py:339: ResourceWarning:
+        # unclosed file <_io.FileIO ... mode='rb' closefd=True>
+        if isinstance(orig_data, BlobFile):
+            orig_data.close()
+
         return value, format_, dimensions
 
 
 @implementer(ITraversable, IPublishTraverse)
 class ImageScaling(BrowserView):
     """ view used for generating (and storing) image scales """
+
     # Ignore some stacks to help with accessing via webdav, otherwise you get a
     # 404 NotFound error.
-    _ignored_stacks = ('manage_DAVget', 'manage_FTPget')
+    _ignored_stacks = ("manage_DAVget", "manage_FTPget")
+    _scale_view_class = ImageScale
 
     def publishTraverse(self, request, name):
         """ used for traversal via publisher, i.e. when using as a url """
-        stack = request.get('TraversalRequestNameStack')
+        stack = request.get("TraversalRequestNameStack")
         image = None
         if stack and stack[-1] not in self._ignored_stacks:
             # field and scale name were given...
@@ -294,27 +301,24 @@ class ImageScaling(BrowserView):
             image = self.scale(name, scale)  # this is an aq-wrapped scale_view
             if image:
                 return image
-        elif '-' in name:
+        elif "-" in name:
             # we got a uid...
-            if '.' in name:
-                name, ext = name.rsplit('.', 1)
+            if "." in name:
+                name, ext = name.rsplit(".", 1)
             storage = AnnotationStorage(self.context)
             info = storage.get(name)
             if info is None:
                 raise NotFound(self, name, self.request)
-            scale_view = ImageScale(self.context, self.request, **info)
+            scale_view = self._scale_view_class(self.context, self.request, **info)
             alsoProvides(scale_view, IStableImageScale)
             return scale_view
         else:
             # otherwise `name` must refer to a field...
-            if '.' in name:
-                name, ext = name.rsplit('.', 1)
+            if "." in name:
+                name, ext = name.rsplit(".", 1)
             value = getattr(self.context, name)
-            scale_view = ImageScale(
-                self.context,
-                self.request,
-                data=value,
-                fieldname=name,
+            scale_view = self._scale_view_class(
+                self.context, self.request, data=value, fieldname=name,
             )
             return scale_view
         raise NotFound(self, name, self.request)
@@ -324,11 +328,8 @@ class ImageScaling(BrowserView):
         # validate access
         value = self.guarded_orig_image(name)
         if not furtherPath:
-            image = ImageScale(
-                self.context,
-                self.request,
-                data=value,
-                fieldname=name,
+            image = self._scale_view_class(
+                self.context, self.request, data=value, fieldname=name,
             )
         else:
             return ImmutableTraverser(self.scale(name, furtherPath[-1]))
@@ -339,12 +340,12 @@ class ImageScaling(BrowserView):
 
     _sizes = {}
 
-    @deprecate('use property available_sizes instead')
+    @deprecate("use property available_sizes instead")
     def getAvailableSizes(self, fieldname=None):
         if fieldname:
-            logger.warn(
-                'fieldname was passed to deprecated getAvailableSizes, but '
-                'will be ignored.',
+            logger.warning(
+                "fieldname was passed to deprecated getAvailableSizes, but "
+                "will be ignored.",
             )
         return self.available_sizes
 
@@ -382,7 +383,7 @@ class ImageScaling(BrowserView):
     def guarded_orig_image(self, fieldname):
         return guarded_getattr(self.context, fieldname, None)
 
-    @deprecate('use getHighPixelDensityScales instead')
+    @deprecate("use getHighPixelDensityScales instead")
     def getRetinaScales(self):
         return getHighPixelDensityScales()
 
@@ -403,7 +404,7 @@ class ImageScaling(BrowserView):
         scale=None,
         height=None,
         width=None,
-        direction='thumbnail',
+        direction="thumbnail",
         **parameters
     ):
         if fieldname is None:
@@ -414,9 +415,9 @@ class ImageScaling(BrowserView):
         if scale is not None:
             if width is not None or height is not None:
                 logger.warn(
-                    'A scale name and width/heigth are given. Those are'
-                    'mutually exclusive: solved by ignoring width/height and '
-                    'taking name',
+                    "A scale name and width/heigth are given. Those are"
+                    "mutually exclusive: solved by ignoring width/heigth and "
+                    "taking name",
                 )
             available = self.available_sizes
             if scale not in available:
@@ -436,7 +437,7 @@ class ImageScaling(BrowserView):
         if info is None:
             return  # 404
 
-        info['srcset'] = self.calculate_srcset(
+        info["srcset"] = self.calculate_srcset(
             fieldname=fieldname,
             height=height,
             width=width,
@@ -445,8 +446,8 @@ class ImageScaling(BrowserView):
             storage=storage,
             **parameters
         )
-        info['fieldname'] = fieldname
-        scale_view = ImageScale(self.context, self.request, **info)
+        info["fieldname"] = fieldname
+        scale_view = self._scale_view_class(self.context, self.request, **info)
         return scale_view
 
     def calculate_srcset(
@@ -455,7 +456,7 @@ class ImageScaling(BrowserView):
         scale=None,
         height=None,
         width=None,
-        direction='thumbnail',
+        direction="thumbnail",
         storage=None,
         **parameters
     ):
@@ -465,27 +466,19 @@ class ImageScaling(BrowserView):
         (orig_width, orig_height) = self.getImageSize(fieldname)
         for hdScale in self.getHighPixelDensityScales():
             # Don't create retina scales larger than the source image.
-            if (
-                (
-                    height and
-                    orig_height and
-                    orig_height < height * hdScale['scale']
-                ) or (
-                    width and
-                    orig_width and
-                    orig_width < width * hdScale['scale']
-                )
+            if (height and orig_height and orig_height < height * hdScale["scale"]) or (
+                width and orig_width and orig_width < width * hdScale["scale"]
             ):
                 continue
-            parameters['quality'] = hdScale['quality']
+            parameters["quality"] = hdScale["quality"]
             scale_src = storage.scale(
                 fieldname=fieldname,
-                height=height * hdScale['scale'] if height else height,
-                width=width * hdScale['scale'] if width else width,
+                height=height * hdScale["scale"] if height else height,
+                width=width * hdScale["scale"] if width else width,
                 direction=direction,
                 **parameters
             )
-            scale_src['scale'] = hdScale['scale']
+            scale_src["scale"] = hdScale["scale"]
             if scale_src is not None:
                 srcset.append(scale_src)
         return srcset
@@ -496,7 +489,7 @@ class ImageScaling(BrowserView):
         scale=None,
         height=None,
         width=None,
-        direction='thumbnail',
+        direction="thumbnail",
         **kwargs
     ):
         scale = self.scale(fieldname, scale, height, width, direction)
@@ -514,11 +507,8 @@ class NavigationRootScaling(ImageScaling):
         )
 
     @ram.cache(_scale_cachekey)
-    def tag(self,
-            brain,
-            fieldname,
-            **kwargs):
+    def tag(self, brain, fieldname, **kwargs):
         obj = brain.getObject()
-        images = obj.restrictedTraverse('@@images')
+        images = obj.restrictedTraverse("@@images")
         tag = images.tag(fieldname, **kwargs)
         return tag
